@@ -53,18 +53,19 @@ class PgnHelper:
         """
         if self.job == 'sort':
             pgnhelper.sort.sort_games(self.inpgnfn, self.outpgnfn,
-                    self.sort_tag, self.sort_direction)
+                    self.sort_tag, self.sort_direction, encoding=self.encoding)
         elif self.job == 'addeco':
             pgnhelper.eco.add_eco(self.inpgnfn, self.outpgnfn,
                     self.inecopgnfn, ply=4, maxply=24)
         elif self.job == 'roundrobin':
-            df = pgnhelper.roundrobin.round_robin(
-                self.inpgnfn,
-                winpoint=self.winpoint,
-                drawpoint=self.drawpoint,
-                armageddonfile=self.armageddonfile,
-                winpointarm=self.winpointarm,
-                losspointarm=self.losspointarm,
-                showmaxscore=self.showmaxscore)
-            pgnhelper.roundrobin.save_roundrobin_table(df,
-                    self.output, self.tablecolor)
+            rr = pgnhelper.roundrobin.RoundRobin(self.inpgnfn,
+                self.armageddonfile, self.winpoint, self.drawpoint,
+                self.winpointarm, self.losspointarm, self.showmaxscore)
+            df = rr.table()
+            rr.save_table(df, self.output, self.tablecolor)
+        elif self.job == 'standing':
+            rr = pgnhelper.roundrobin.RoundRobin(self.inpgnfn,
+                self.armageddonfile, self.winpoint, self.drawpoint,
+                self.winpointarm, self.losspointarm, self.showmaxscore)
+            df = rr.standing()
+            rr.save_table(df, self.output, self.tablecolor)
